{-
Copyright (c) 2017 Ryan Olson

GNU GENERAL PUBLIC LICENSE
    Version 3, 29 June 2007

This file is part of elm-time-range.

elm-time-range is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

elm-time-range is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with elm-time-range.  If not, see <http://www.gnu.org/licenses/>.
-}

module Tests exposing (..)

import Date exposing (Date)
import Expect
import Test exposing (..)
import Time
import TimeRange


all : Test
all =
    describe "Range"
        [ timeRange
        ]


timeRange : Test
timeRange =
    describe "Time Range"
        [ test "Create time range and access times" <|
            \() ->
                let
                    startTime =
                        "09/01/2016 05:00 PM UTC"
                            |> dateFromString
                            |> Date.toTime

                    endTime =
                        "09/01/2016 07:00 PM UTC"
                            |> dateFromString
                            |> Date.toTime

                    timeRange =
                        TimeRange.fromStartAndEnd startTime endTime
                in
                    timeRange
                        |> Expect.all
                            [ (\timeRange ->
                                Expect.equal startTime (TimeRange.start timeRange)
                              )
                            , (\timeRange ->
                                Expect.equal endTime (TimeRange.end timeRange)
                              )
                            ]
        , test "within with actual time inside time range" <|
            \() ->
                let
                    startTime =
                        "09/01/2016 05:00 PM UTC"
                            |> dateFromString
                            |> Date.toTime

                    endTime =
                        "09/01/2016 07:00 PM UTC"
                            |> dateFromString
                            |> Date.toTime

                    containedTime =
                        "09/01/2016 06:00 PM UTC"
                            |> dateFromString
                            |> Date.toTime

                    timeRange =
                        TimeRange.fromStartAndEnd startTime endTime
                in
                    TimeRange.isContainedWithin containedTime timeRange
                        |> Expect.true "containedTime should be within timeRange"
        , test "isContainedWithin with time outside of time range" <|
            \() ->
                let
                    startTime =
                        "09/01/2016 05:00 PM UTC"
                            |> dateFromString
                            |> Date.toTime

                    endTime =
                        "09/01/2016 07:00 PM UTC"
                            |> dateFromString
                            |> Date.toTime

                    laterTime =
                        "09/01/2016 10:00 PM UTC"
                            |> dateFromString
                            |> Date.toTime

                    timeRange =
                        TimeRange.fromStartAndEnd startTime endTime
                in
                    TimeRange.isContainedWithin laterTime timeRange
                        |> Expect.false "laterTime should not be within timeRange"
        , test "overlap with valid overlap" <|
            \() ->
                let
                    timeRange1 =
                        timeRangeFrom "09/01/2016 05:00 PM UTC" "09/01/2016 07:00 PM UTC"

                    timeRange2 =
                        timeRangeFrom "09/01/2016 06:00 PM UTC" "09/01/2016 08:00 PM UTC"
                in
                    TimeRange.overlaps timeRange1 timeRange2
                        |> Expect.true "time ranges should overlap"
        , test "overlap with no overlap" <|
            \() ->
                let
                    timeRange1 =
                        timeRangeFrom "09/01/2016 05:00 PM UTC" "09/01/2016 07:00 PM UTC"

                    timeRange2 =
                        timeRangeFrom "09/01/2016 08:00 PM UTC" "09/01/2016 10:00 PM UTC"
                in
                    TimeRange.overlaps timeRange1 timeRange2
                        |> Expect.false "time ranges should not overlap"
        , test "difference" <|
            \() ->
                let
                    timeRange =
                        timeRangeFrom "09/01/2016 05:00 PM UTC" "09/01/2016 07:00 PM UTC"
                in
                    TimeRange.difference timeRange
                        |> Time.inMinutes
                        |> Expect.equal 120
        ]


timeRangeFrom : String -> String -> TimeRange.TimeRange
timeRangeFrom startTimeString endTimeString =
    TimeRange.fromStartAndEnd
        (Date.toTime <|
            dateFromString startTimeString
        )
        (Date.toTime <|
            dateFromString endTimeString
        )


dateFromString : String -> Date
dateFromString s =
    Result.withDefault (Date.fromTime 0) (Date.fromString s)
